## this is the replication R code for Cao 2010: Networks As Channels of Policy Diffusion: Explaining Worldwide Changes in Capital Taxation, 1998-2006. 
## the data needed to run the following code are posted in the author's website: privatewww.essex.ac.uk/~caox
## there, you will find detailed instruction there on how to download the data and run the replication code. 


dd <- c("C:/XunCao/Research/Desert/Tax_Exp/data") 
setwd(dd)

####
# DVs: tax on corporate
Tax_corp<-dget("DV/Tax_corp_wdi9806")

oecd<-c("AUL", "AUS", "BEL", "CAN", "CZR", "DEN", "FIN", "FRN", "GMY", "GRC", 
        "HUN", "ICE", "IRE", "ITA", "JPN", "ROK", "LUX", "MEX", "NTH", "NEW",
        "NOR", "POL", "POR", "SLO", "SPN", "SWD", "SWZ", "TUR", "UKG", "USA")
N<-NULL
M<-NULL
for (j in 1:dim(Tax_corp)[2]){m<-mean(Tax_corp[,j], na.rm=T); M<-c(M, m)}
plot(1998:2006, M, xlim=c(1997.5, 2006.9), ylim=c(23, 36), pch=19, type="p", xlab="", ylab="", cex=1.5)
N<-rbind(N, M)

M<-NULL
for (j in 1:dim(Tax_corp)[2]){m<-mean(Tax_corp[oecd,j], na.rm=T); M<-c(M, m)}
points(1998:2006, M, type="p", pch=19, col="gray", cex=1.5)
N<-rbind(N, M)

M<-NULL
for (j in 1:dim(Tax_corp)[2]){m<-mean(Tax_corp[setdiff(rownames(Tax_corp), oecd),j], na.rm=T); M<-c(M, m)}
points(1998:2006, M, col="black", type="p", cex=1.5)
N<-rbind(N, M)

text(2004, 35, "OECD Countries", pos=4, cex=1.2)
text(2004, 34, "All Countries", pos=4, cex=1.2)
text(2004, 33, "Non-OECD Countries", pos=4, cex=1.2)

points(2003.8, 35, pch=19, cex=1.5, col="gray")
points(2003.8, 34, pch=19, cex=1.5, col="black")
points(2003.8, 33, cex=1.5, col="black")

colnames(N)<-as.character(1998:2006)
library(xtable)
xtable(round(N, digits=1))




##### now, let's model:
dv.tax.c<-year<-cty<-NULL
for (j in 1:dim(Tax_corp)[2]){dv.tax.c<-c(dv.tax.c, Tax_corp[, j])
                              year<-c(year, rep(c(1998:2006)[j], dim(Tax_corp)[1]))
                              cty<-c(cty, rownames(Tax_corp))
                              }

Tax_corp_lg<-Tax_corp
Tax_corp_lg[,4]<-Tax_corp_lg[,3]
Tax_corp_lg[,8]<-Tax_corp_lg[,7]
                              
dv.tax.c.lg<-rep(NA, dim(Tax_corp_lg)[1])
for (j in 1:(dim(Tax_corp_lg)[2]-1)){dv.tax.c.lg<-c(dv.tax.c.lg, Tax_corp_lg[, j])}


# Add in nodal characteristics:
# inflation rate
# GDP growth
# umemployment rate
# age dependency
# financial openness (legal)
# trade openness
# public debt (missing values)


## Inflation rate
Infl<-dget("Node/weoINFL8005"); Infl[, dim(Infl)[2]]<-as.numeric(Infl[, dim(Infl)[2]])
infl<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Infl) & as.character(year[i]) %in% colnames(Infl)){infl<-c(infl, Infl[cty[i], as.character(year[i])])}
                              else {infl<-c(infl, NA)}}
infl.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Infl) & as.character(year[i]-1) %in% colnames(Infl)){infl.lg<-c(infl.lg, Infl[cty[i], as.character(year[i]-1)])}
                              else {infl.lg<-c(infl.lg, NA)}
                              }

## DGP growth
Grow<-dget("Node/WDI_GDPgrow6105")
Temp<-Grow
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
grow<-iv; grow.lg<-iv.lg


# GDP per capita
GDDPPC<-dget("Node/weoGDPPC8005")
Temp<-GDDPPC
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
gdppc<-as.numeric(iv); gdppc.lg<-as.numeric(iv.lg)
gdppc<-gdppc/1000; gdppc.lg<-gdppc.lg/1000

## Unemployment rate
Unemp<-dget("Node/un_wdi9006") # there are lots of missing values here.
Temp<-Unemp
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
unemp<-iv; unemp.lg<-iv.lg


## Net government debt
Debt<-dget("Node/weoNDEBT8005") # there are lots of missing values here.
Temp<-Debt
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
debt<-iv; debt.lg<-iv.lg


## Age dependency
DP65<-dget("Node/d65_wdi9006")
Temp<-DP65
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
dp65<-iv; dp65.lg<-iv.lg

DP14<-dget("Node/d14_wdi9006")
Temp<-DP14
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
dp14<-iv; dp14.lg<-iv.lg


## Financial Openness:
CapOpen<-dget("Node/CapOpen_extd7005")
Temp<-CapOpen
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
capopen<-iv; capopen.lg<-iv.lg


## Trade Openness:
TradeOpen<-dget("Node/DOT_WEO_tradeopen9005")
Temp<-TradeOpen
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
tradeopen<-iv; tradeopen.lg<-iv.lg




## a non-network model:
fit.node<-lm(dv.tax.c~dv.tax.c.lg
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +as.factor(cty)
                      +as.factor(year)
                      )
summary(fit.node)
                      


######### Add in Geography and Common Language ##########

## 1: inverse distance between capital cities:

distance<-dget("http://privatewww.essex.ac.uk/~caox/publications/wsc_replicate/distance") # in 1000 km
diag(distance)<-NA; 
w_dst<-(1/distance)/rowSums(1/distance, na.rm=T); w_dst[is.na(w_dst)]<-0

w<-matrix(0, ncol=length(rownames(Tax_corp)), nrow=length(rownames(Tax_corp)))
colnames(w)<-rownames(w)<-rownames(Tax_corp)

for (i in 1:dim(w_dst)[1])      {
for (j in (1:dim(w_dst)[2])[-i]){if (rownames(w_dst)[i] %in% rownames(Tax_corp) & colnames(w_dst)[j] %in% rownames(Tax_corp))
                                    {w[rownames(w_dst)[i], colnames(w_dst)[j]]<-w_dst[i, j]}
                                   }}

Neib.cap<-rep(NA, dim(w)[1])
year.lag<-c(1998, 1999, 2000, 2000, 2002, 2003, 2004, 2004) 

Tax<-Tax_corp
Tax[is.na(Tax)]<-0
for (k in 1:length(year.lag)){neib.cap<-w%*%Tax[, as.character(year.lag[k])]
                              Neib.cap<-c(Neib.cap, neib.cap)
                              }
                              
fit.cap<-lm(dv.tax.c~
                      as.factor(cty)
                      +as.factor(year)
                      +dv.tax.c.lg
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.cap
                      )
summary(fit.cap)



# 2.  or use smallest distance, this is similar to the COW, but I have a more detailed distance measurement. 
### I think this is more appropriate for the diffusion process rather than use the distance between capital cities. 

# for Gleditsch and Ward 2001: 
# Briefly stated, we have opted for the basic principle of
# measuring the shortest distance between the two closest physical locations for every pair of
# independent polities between 1875 and
# 1996. The database records the shortest distance in statute kilometers between points on
# the outer boundaries for two polities, regardless ofwhether the states are separated by land
# or sea, given the borders in place in a particular time period. Actual shared borders are
# coded as having distances of zero. In our view, this database provides an integrated frame-
# work that combines the advantages of binary contiguity data and more disaggregated continuous distance data.

# read in the minimum distance data: notice this only include country pairs that are close enough to each other:
# that is, within 950km between the closest points between them
# 0 means "strict" contiguity.
# meaning we cannot use inverse distance for this data set: there is no record for those having a distance >950km.
# or we can use inverse distance for those within 950 km and assume that those beyond 950 have no effect at all.

mdd<-read.csv("W/distance/smallmdd.csv", as.is=T)
mdd$ida[mdd$ida=="GFR"]<-"GMY"; mdd$idb[mdd$idb=="GFR"]<-"GMY"
mdd$ida[mdd$ida=="YUG"]<-"SER"; mdd$idb[mdd$idb=="YUG"]<-"SER"

mdd98 <- na.omit(subset(mdd, mdd$year>=1998)) 

wmdd<-matrix(NA, ncol=length(rownames(Tax_corp)),nrow=length(rownames(Tax_corp)))
rownames(wmdd)<-colnames(wmdd)<-rownames(Tax_corp)

for (i in 1:dim(mdd98)[1]) {if (mdd98$ida[i] %in% rownames(Tax_corp) & mdd98$idb[i] %in% rownames(Tax_corp))
                                {wmdd[mdd98$ida[i],mdd98$idb[i]] <- mdd98$mindist[i]}}

# notice that there are large amounts of NA: could be missing values,
#                                            could be the distance between two countries's closest points of their outer 
#                                            boundaries are bigger than 950 km. 

# strick contiguity: share land border
thres<-950
w<-wmdd
w[w>thres]<-NA
w[w<=thres]<-1; 
w[is.na(w)]<-0

sm<-rowSums(w)
sm[sm==0]<-1

w<-w/sm

Neib.mdd<-rep(NA, dim(w)[1])
year.lag<-c(1998, 1999, 2000, 2000, 2002, 2003, 2004, 2004) 

Tax<-Tax_corp
Tax[is.na(Tax)]<-0
for (k in 1:length(year.lag)){
neib.mdd<-w%*%Tax[, as.character(year.lag[k])] 
Neib.mdd<-c(Neib.mdd, neib.mdd)
}

fit.space<-lm(dv.tax.c~
                      dv.tax.c.lg
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.mdd
                      + as.factor(cty)
                      +as.factor(year))
summary(fit.space)
# get the design matrix


cc<-as.numeric(rownames(model.matrix(fit.space)))

fit.space<-lm(dv.tax.c[cc]~
                      dv.tax.c.lg[cc]
                      +infl[cc]
                      +grow[cc]
                      +unemp[cc]
                      +I(dp65+dp14)[cc]
                      +gdppc[cc]
                      +capopen[cc]
                      +tradeopen[cc]
                      +Neib.mdd[cc]
                      #+ as.factor(cty)[cc]
                      #+as.factor(year)[cc]
                      )
summary(fit.space)




## do a loop for different thresholds:
Thres<-c(0, 100, 200, 300, 400, 500, 600, 700, 800, 900, 950)
CUT<-NULL

for (i in 1:length(Thres)){
thres<-Thres[i]
w<-wmdd
w[w>thres]<-NA
w[w<=thres]<-1; 
w[is.na(w)]<-0

sm<-rowSums(w)
sm[sm==0]<-1

w<-w/sm

Neib.mdd<-rep(NA, dim(w)[1])
year.lag<-c(1998, 1999, 2000, 2000, 2002, 2003, 2004, 2004) 

Tax<-Tax_corp
Tax[is.na(Tax)]<-0
for (k in 1:length(year.lag)){
neib.mdd<-w%*%Tax[, as.character(year.lag[k])] 
Neib.mdd<-c(Neib.mdd, neib.mdd)
}

fit.space<-lm(dv.tax.c~
                      as.factor(cty)
                      +as.factor(year)
                      +dv.tax.c.lg
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.mdd
                      )
# summary(fit.space)
CUT<-rbind(CUT, c(thres, summary(fit.space)$coef["Neib.mdd",]))
}

va<-as.character(CUT[,1])

op.m<-CUT[,2]
op.se<-CUT[,3]

par(mfrow=c(1,1))
xlim <- c(1,11)
ylim <- c(-.1,.5)

plot(xlim,ylim,type="n",xlab="Distance Threshold",ylab="",axes=FALSE, main="");abline(h=0, col="gray")
axis(2);  axis(1, labels=va, at=c(c(1:11)-.1), las=1, padj=0, hadj=0, col="lightgray", tick=F)
segments(1:11, op.m-1.645*op.se, 1:11, op.m+1.645*op.se, col="gray", lwd=2); points(1:11, op.m, pch=19, cex=1)



## 3. Distance from COW:
w_dst<-dget("W/distance/w_dist")

w<-matrix(0, ncol=length(rownames(Tax_corp)), nrow=length(rownames(Tax_corp)))
colnames(w)<-rownames(w)<-rownames(Tax_corp)

for (i in 1:dim(w_dst)[1])      {
for (j in (1:dim(w_dst)[2])[-i]){if (rownames(w_dst)[i] %in% rownames(Tax_corp)  & colnames(w_dst)[j] %in% rownames(Tax_corp))
                                  {w[rownames(w_dst)[i], colnames(w_dst)[j]]<-w_dst[i, j]}
                                   }}
w[w>1]<-1 

# use the lowest standard --- Separated by 400 miles of water or less 
# 1 Separated by a land or river border 
# 2 Separated by 12 miles of water or less 
# 3 Separated by 24 miles of water or less 
# 4 Separated by 150 miles of water or less 
# 5 Separated by 400 miles of water or less 

rs<-rowSums(w, na.rm=T)
rs[rs==0]<-1

w<-w/rs # notice this is row-standardize which results in non-symmetric version:

Neib.dst<-rep(NA, dim(w)[1])# the first year's temporarily lagged spatial lag is NA
year.lag<-c(1998, 1999, 2000, 2000, 2002, 2003, 2004, 2004) # notice the data for 2001 and 2005 are missing except for one country
                                                            # we substitude 2000 for 2001, and 2004 for 2005, but only for temporal
                                                            # lag and temporally lagged spatial lag. 
Tax<-Tax_corp
Tax[is.na(Tax)]<-0
for (k in 1:length(year.lag)){
neib.dst<-w%*%Tax[, as.character(year.lag[k])] 
Neib.dst<-c(Neib.dst, neib.dst)
}

fit.space<-lm(dv.tax.c~dv.tax.c.lg
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.dst
                      +as.factor(cty)
                      +as.factor(year)
                      )
summary(fit.space)
    
  
                 
## Common language:
w_lng<-dget("W/culture/w_clang")

wl<-matrix(0, ncol=length(rownames(Tax_corp)), nrow=length(rownames(Tax_corp)))
colnames(wl)<-rownames(wl)<-rownames(Tax_corp)

for (i in 1:dim(w_lng)[1])      {
for (j in (1:dim(w_lng)[2])[-i]){if (rownames(w_lng)[i] %in% rownames(Tax_corp)  & colnames(w_lng)[j] %in% rownames(Tax_corp))
                                  {wl[rownames(w_lng)[i], colnames(w_lng)[j]]<-w_lng[i, j]}
                                   }}
rs<-rowSums(wl, na.rm=T)
rs[rs==0]<-1

wl<-wl/rs # notice this is row-standardize which results in non-symmetric version:

Neib.lng<-rep(NA, dim(wl)[1])# the first year's temporarily lagged spatial lag is NA

year.lag<-c(1998, 1999, 2000, 2000, 2002, 2003, 2004, 2004) 


for (k in 1:length(year.lag)){
neib.lng<-wl%*%Tax[, as.character(year.lag[k])] 
Neib.lng<-c(Neib.lng, neib.lng)
}

# notice common language and contiguity are not that correlated;

fit.lng<-lm(dv.tax.c~dv.tax.c.lg
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                       +Neib.mdd
                      +Neib.lng
                      +as.factor(cty)
                      +as.factor(year)
                      )
summary(fit.lng)

cc<-as.numeric(rownames(model.matrix(fit.lng)))

fit.lng<-lm(dv.tax.c[cc]~#dv.tax.c.lg
                     # +infl
                     # +grow
                     # +unemp
                     # +I(dp65+dp14)
                     # +gdppc
                     # +capopen
                     # +tradeopen
                     #  +Neib.mdd
                     # +Neib.lng
                      +as.factor(cty)[cc]
                      +as.factor(year)[cc]
                      )
summary(fit.lng)




#### Move to Networks: portfolio, FDI, trade, IGO ####

# 1. portfolio:
p2001<-dget("W/portfolio/cor.portf2001")
p2002<-dget("W/portfolio/cor.portf2002")
p2003<-dget("W/portfolio/cor.portf2003")
p2004<-dget("W/portfolio/cor.portf2004")
p2005<-dget("W/portfolio/cor.portf2005")
Portf<-list(p2001, p2002, p2003, p2004, p2005)
year.lag<-c(2000, 2002, 2003, 2004, 2004)


CUT<-NULL
for (m in c(seq(0,.95, by=.05))) 
{
Cn<-NULL
Neib.portf<-rep(NA, 4*dim(Tax_corp)[1]) # we don't have data for 1998-2001: notice spatial lags are temporarily lagged also.
for (i in 1:5){se<-Portf[[i]]
               diag(se)<-0
               # se<-se+1
               se[se<=m]<-0 # this is to use a cutpoint in correlations to specify close neighbors.
               sm<-rowSums(se);
               sm[sm==0]<-1
               se<-se/sm 
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               
               counts<-se[se.ctry, se.ctry]
               counts[counts>0]<-1
               Cn<-cbind(Cn, colSums(counts))
               
               n.se<-se[se.ctry, se.ctry]%*%Tax[se.ctry, as.character(year.lag)[i]]
               nse<-NULL
               for (j in 1:dim(Tax_corp)[1]){if (rownames(Tax_corp)[j] %in% rownames(n.se)){nse<-c(nse, n.se[rownames(Tax_corp)[j],])}
                                             else {nse<-c(nse, NA)}}
               # nse[nse==0]<-mean(Tax[se.ctry, as.character(year.lag)[i]], na.rm=T)
               Neib.portf<-c(Neib.portf, nse)
               }
fit.port<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                       +gdppc
                      +capopen
                      +tradeopen
                      +Neib.mdd
                      #+Neib.lng
                      +Neib.portf
                     )
CUT<-rbind(CUT, c(m, summary(fit.port)$coef["Neib.portf",]))
}
CUT    

Cn<-NULL
Neib.portf<-rep(NA, 4*dim(Tax_corp)[1]) # we don't have data for 1998-2001: notice spatial lags are temporarily lagged also.
for (i in 1:5){se<-Portf[[i]]
               diag(se)<-0
               # se<-se+1
               se[se<=.5]<-0 # this is to use a cutpoint in correlations to specify close neighbors.
               sm<-rowSums(se);
               sm[sm==0]<-1
               se<-se/sm 
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               
               counts<-se[se.ctry, se.ctry]
               counts[counts>0]<-1
               Cn<-cbind(Cn, colSums(counts))
               
               n.se<-se[se.ctry, se.ctry]%*%Tax[se.ctry, as.character(year.lag)[i]]
               nse<-NULL
               for (j in 1:dim(Tax_corp)[1]){if (rownames(Tax_corp)[j] %in% rownames(n.se)){nse<-c(nse, n.se[rownames(Tax_corp)[j],])}
                                             else {nse<-c(nse, NA)}}
               # nse[nse==0]<-mean(Tax[se.ctry, as.character(year.lag)[i]], na.rm=T)
               Neib.portf<-c(Neib.portf, nse)
               }
               
xtable(matrix(names(colSums(counts)), ncol=5))

fit.port<-lm(dv.tax.c[!is.na(Neib.portf)]~dv.tax.c.lg[!is.na(Neib.portf)]
                      +as.factor(cty[!is.na(Neib.portf)])
                      +as.factor(year[!is.na(Neib.portf)])
                      +infl[!is.na(Neib.portf)]
                      +grow[!is.na(Neib.portf)]
                      +unemp[!is.na(Neib.portf)]
                      +I(dp65[!is.na(Neib.portf)]+dp14[!is.na(Neib.portf)])
                       +gdppc[!is.na(Neib.portf)]
                      +capopen[!is.na(Neib.portf)]
                      +tradeopen[!is.na(Neib.portf)]
                       +Neib.mdd[!is.na(Neib.portf)]
                      #+Neib.lng
                       +Neib.portf[!is.na(Neib.portf)]
                     )
                                      
summary(fit.port)



### take a look at the spill over from one country:
i<-1 # 2001 
delta.unemp<-10
coef.unemp <-summary(fit.port)$coef["unemp[!is.na(Neib.portf)]", ][1]
coef.portf <-summary(fit.port)$coef["Neib.portf[!is.na(Neib.portf)]", ][1]
## need to add standard error for the spatial coef here: 90%
coef.portf.low90  <-coef.portf - 1.65*summary(fit.port)$coef["Neib.portf[!is.na(Neib.portf)]", ][2]
coef.portf.high90 <-coef.portf + 1.65*summary(fit.port)$coef["Neib.portf[!is.na(Neib.portf)]", ][2]


se<-Portf[[i]]  # this is 2001
               diag(se)<-0
               se[se<=.5]<-0 
               sm<-rowSums(se);
               sm[sm==0]<-1
               se<-se/sm 
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))

# let's try Austria:
               base<-Tax[se.ctry, as.character(year.lag)[i]]
               names(base)<-se.ctry
               base[names(base)=="AUS"]<-delta.unemp*coef.unemp
               base[names(base)!="AUS"]<-0            
               n.se<-coef.portf*se[se.ctry, se.ctry]%*%base
               
               n.se.low90 <-coef.portf.low90 *se[se.ctry, se.ctry]%*%base
               n.se.high90<-coef.portf.high90*se[se.ctry, se.ctry]%*%base
               
n.se[rownames(n.se)=="AUS"]<-delta.unemp*coef.unemp
# n.se<--1*n.se
# n.se<-round(n.se, digits=2)

N.se<-cbind(n.se.low90, n.se, n.se.high90) # this is the changes in 2002
round(N.se, digits=c(2, 2, 2))


# but for the long term effects at t+1, t+2 ...
se<-Portf[[i+1]] # 2002
               diag(se)<-0
               se[se<=.5]<-0 
               sm<-rowSums(se);
               sm[sm==0]<-1
               se<-se/sm 
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               se.ctry<-intersect(se.ctry, rownames(N.se))
               
               
 
# now it is not just Austria:
               base<-N.se[se.ctry, 2]            
               n.se<-coef.portf*se[se.ctry, se.ctry]%*%as.vector(base)
               
               n.se.low90 <-coef.portf.low90 *se[se.ctry, se.ctry]%*%as.vector(base)
               n.se.high90<-coef.portf.high90*se[se.ctry, se.ctry]%*%as.vector(base)

N.se<-cbind(n.se.low90, n.se, n.se.high90) # this is for 2003
round(N.se, digits=c(2, 3, 2))


# we can continue this long-term effects. 


# what about if the whole world is just composed of these 133 countries.
Cn<-NULL
Neib.portf<-rep(NA, 4*dim(Tax_corp)[1]) # we don't have data for 1998-2001: notice spatial lags are temporarily lagged also.
for (k in 1:5){se<-Portf[[k]]

               wp<-matrix(NA, ncol=length(rownames(Tax_corp)), nrow=length(rownames(Tax_corp)))
               colnames(wp)<-rownames(wp)<-rownames(Tax_corp)

               for (i in 1:dim(se)[1])      {
               for (j in (1:dim(se)[2])[-i]){if (rownames(se)[i] %in% rownames(Tax_corp)  & colnames(se)[j] %in% rownames(Tax_corp))
                                                 {wp[rownames(se)[i], colnames(se)[j]]<-se[i, j]}
                                   }}

               diag(wp)<-0
               #se<-se+1
               wp[wp<=0.5]<-0 # this is to use a cutpoint in correlations to specify close neighbors.
               wp[is.na(wp)]<-0
               sm<-rowSums(wp);
               sm[sm==0]<-1
               wp<-wp/sm 
                              
               counts<-wp
               counts[counts>0]<-1
               Cn<-cbind(Cn, colSums(counts))
               
               n.se<-wp%*%Tax[, as.character(year.lag)[k]]
               Neib.portf<-c(Neib.portf, n.se)
               }
               
               
fit.port<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      # +Neib.dst
                      #+Neib.lng
                      +Neib.portf
                     )
summary(fit.port)




# what about we use k-nearest neighborhood

k<-15

CUT<-NULL

for (k in 1:80){

Neib.portf<-rep(NA, 4*dim(Tax_corp)[1]) 

for (i in 1:5){se<-Portf[[i]]
               diag(se)<-0
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               se<-se[se.ctry, se.ctry]
               for (j in 1:dim(se)[1]){se[j,][se[j,]<sort(se[j, ], decreasing = T)[k]]<-0}
               sm<-rowSums(se)
               sm[sm==0]<-1
               se<-se/sm 
               n.se<-se%*%Tax[se.ctry, as.character(year.lag)[i]]
               nse<-NULL
               for (j in 1:dim(Tax_corp)[1]){if (rownames(Tax_corp)[j] %in% rownames(n.se)){nse<-c(nse, n.se[rownames(Tax_corp)[j],])}
                                             else {nse<-c(nse, NA)}}
               Neib.portf<-c(Neib.portf, nse)
               }
# plot(Neib.portf, dv.tax.c) 

fit.port<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.mdd
                      #+Neib.lng
                      +Neib.portf
                     )
CUT<-rbind(CUT, c(k, summary(fit.port)$coef["Neib.portf",]))
}
CUT

postscript("rho_k.ps", width = 4.0, height = 4.0,
                horizontal = T, 
                family = "Times")

plot(1:100, 1:100, ylim=c(-.7, 1.6), xlim=c(1, 80), cex.axis=1, las=0, xaxp=c(0,80,8),
     type="n", xlab="Choices of K", ylab="Estimates for Spatial Lag"); abline(h=0, col="gray")
lines(1:80, CUT[,2], lwd=2)
lines(1:80, CUT[,2]-1.6*CUT[,3], col="gray", lwd=2)
lines(1:80, CUT[,2]+1.64*CUT[,3], col="gray", lty=2, lwd=2)

dev.off()

                
summary(fit.port)



### 2. let's take a look at FDI:
fdi.f1998<-dget("W/fdi/fdi_oecd_un_f_1998")
fdi.f1999<-dget("W/fdi/fdi_oecd_un_f_1999")
fdi.f2000<-dget("W/fdi/fdi_oecd_un_f_2000")
fdi.f2001<-dget("W/fdi/fdi_oecd_un_f_2001")
fdi.f2002<-dget("W/fdi/fdi_oecd_un_f_2002")

fdi.s1998<-dget("W/fdi/fdi_oecd_un_1998")
fdi.s1999<-dget("W/fdi/fdi_oecd_un_1999")
fdi.s2000<-dget("W/fdi/fdi_oecd_un_2000")
fdi.s2001<-dget("W/fdi/fdi_oecd_un_2001")
fdi.s2002<-dget("W/fdi/fdi_oecd_un_2002")


c.f.fdi.1998<-cor(fdi.f1998, use="pairwise.complete.obs") 
c.f.fdi.1999<-cor(fdi.f1999, use="pairwise.complete.obs") 
c.f.fdi.2000<-cor(fdi.f2000, use="pairwise.complete.obs") 
c.f.fdi.2001<-cor(fdi.f2001, use="pairwise.complete.obs") 
c.f.fdi.2002<-cor(fdi.f2002, use="pairwise.complete.obs") 

c.s.fdi.1998<-cor(fdi.s1998, use="pairwise.complete.obs") 
c.s.fdi.1999<-cor(fdi.s1999, use="pairwise.complete.obs") 
c.s.fdi.2000<-cor(fdi.s2000, use="pairwise.complete.obs") 
c.s.fdi.2001<-cor(fdi.s2001, use="pairwise.complete.obs") 
c.s.fdi.2002<-cor(fdi.s2002, use="pairwise.complete.obs") 

FDI<-list(c.f.fdi.1998, 
          c.f.fdi.1999,
          c.f.fdi.2000, 
          c.f.fdi.2001, 
          c.f.fdi.2002)
year.lag<-c(1998, 1999, 2000, 2000, 2002) 


CUT<-NULL
for (m in c(seq(0,.95, by=.05))) 
{
Cn<-NULL

Neib.fdi<-rep(NA, 1*dim(Tax_corp)[1]) # we don't have data for 1998's lag, neither for year 2004-2006

for (i in 1:5){se<-FDI[[i]]; se[is.na(se)]<-0; diag(se)<-0
               se[se<=m]<-0 # this is to use a cutpoint in correlations to specify close neighbors.
               sm<-rowSums(se);
               sm[sm==0]<-1
               se<-se/sm 
               # se<-se/rowSums(se) 
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               n.se<-se[se.ctry, se.ctry]%*%Tax[se.ctry, as.character(year.lag)[i]]
               nse<-NULL
               for (j in 1:dim(Tax_corp)[1]){if (rownames(Tax_corp)[j] %in% rownames(n.se)){nse<-c(nse, n.se[rownames(Tax_corp)[j],])}
                                             else {nse<-c(nse, NA)}}
               Neib.fdi<-c(Neib.fdi, nse)
               }
               
Neib.fdi<-c(Neib.fdi, rep(NA, 3*dim(Tax_corp)[1])) # we don't have data for 1998-2001: notice spatial lags are temporarily lagged also.

fit.fdi<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.dst
                      # +Neib.lng
                      # +Neib.portf
                      +Neib.fdi
                     )
CUT<-rbind(CUT, c(m, summary(fit.fdi)$coef["Neib.fdi",]))
}
CUT   

summary(fit.fdi)

plot(Neib.fdi, dv.tax.c) 


# what about we use k-nearest neighborhood
CUT<-NULL

for (k in 1:80){

Neib.fdi<-rep(NA, 1*dim(Tax_corp)[1]) # we don't have data for 1998's lag, neither for year 2004-2006
k<-21
for (i in 1:5){se<-FDI[[i]]; se[is.na(se)]<-0; diag(se)<-0
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               se<-se[se.ctry, se.ctry]
               for (j in 1:dim(se)[1]){se[j,][se[j,]<sort(se[j, ], decreasing = T)[k]]<-0}
               se<-se/rowSums(se) 
               n.se<-se%*%Tax[se.ctry, as.character(year.lag)[i]]
               nse<-NULL
               for (j in 1:dim(Tax_corp)[1]){if (rownames(Tax_corp)[j] %in% rownames(n.se)){nse<-c(nse, n.se[rownames(Tax_corp)[j],])}
                                             else {nse<-c(nse, NA)}}
               Neib.fdi<-c(Neib.fdi, nse)
               }
Neib.fdi<-c(Neib.fdi, rep(NA, 3*dim(Tax_corp)[1])) # we don't have data for 1998-2001: notice spatial lags are temporarily lagged also.

fit.fdi<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.mdd
                      # +Neib.lng
                      # +Neib.portf
                      +Neib.fdi
                     )
                     
CUT<-rbind(CUT, c(k, summary(fit.fdi)$coef["Neib.fdi",]))
}
CUT


plot(Neib.portf, dv.tax.c) 



### 3. what about structural equivalence in trade:
### data are available until 2000: used as lags, we have 1998, 1999 for only two years.
seexp.1998<-dget("W/seexp/expsq_acro98")
seexp.1999<-dget("W/seexp/expsq_acro99")
SE<-list(seexp.1998, seexp.1999)
year.lag<-c(1998, 1999)

Neib.seexp<-rep(NA, 1*dim(Tax_corp)[1]) # we don't have data for 1998's lag, neither for year 2001-2006
for (i in 1:2){se<-SE[[i]]; se[is.na(se)]<-0; diag(se)<-0
               se[se<=0.5]<-0 # this is to use a cutpoint in correlations to specify close neighbors.
               se<-se/rowSums(se) 
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               n.se<-se[se.ctry, se.ctry]%*%Tax[se.ctry, as.character(year.lag)[i]]
               nse<-NULL
               for (j in 1:dim(Tax_corp)[1]){if (rownames(Tax_corp)[j] %in% rownames(n.se)){nse<-c(nse, n.se[rownames(Tax_corp)[j],])}
                                             else {nse<-c(nse, NA)}}
               Neib.seexp<-c(Neib.seexp, nse)
               }
               
Neib.seexp<-c(Neib.seexp, rep(NA, 6*dim(Tax_corp)[1])) 

fit.seexp<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.dst
                      # +Neib.lng
                      # +Neib.fdi
                      +Neib.seexp
                     )
summary(fit.seexp)


CUT<-NULL

for (k in 1:60){

# k<-21
Neib.seexp<-rep(NA, 1*dim(Tax_corp)[1]) # we don't have data for 1998's lag, neither for year 2001-2006
for (i in 1:2){se<-SE[[i]]; se[is.na(se)]<-0; diag(se)<-0
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               se<-se[se.ctry, se.ctry]
               for (j in 1:dim(se)[1]){se[j,][se[j,]<sort(se[j, ], decreasing = T)[k]]<-0}
               sm<-rowSums(se)
               sm[sm==0]<-1
               se<-se/sm 
               n.se<-se%*%Tax[se.ctry, as.character(year.lag)[i]]
               nse<-NULL
               for (j in 1:dim(Tax_corp)[1]){if (rownames(Tax_corp)[j] %in% rownames(n.se)){nse<-c(nse, n.se[rownames(Tax_corp)[j],])}
                                             else {nse<-c(nse, NA)}}
               Neib.seexp<-c(Neib.seexp, nse)
               }
Neib.seexp<-c(Neib.seexp, rep(NA, 6*dim(Tax_corp)[1])) 

fit.seexp<-lm(dv.tax.c~dv.tax.c.lg
                      #+as.factor(cty)
                      #+as.factor(year)
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      #+Neib.mdd
                      # +Neib.lng
                      # +Neib.fdi
                      +Neib.seexp
                     )

CUT<-rbind(CUT, c(k, summary(fit.seexp)$coef["Neib.seexp",]))
}
CUT







## Wednesday, 20.May.2009
## let's use the comtrade data: 

seexp<-dget("W/seexp/expsq_acro98")

seexp.1998<-dget("W/seexp/cor_sitc_stack219_1998")#[colnames(seexp)[-30], colnames(seexp)[-30]] # get rid of "KAK"
seexp.1999<-dget("W/seexp/cor_sitc_stack219_1999")#[colnames(seexp)[-30], colnames(seexp)[-30]]
seexp.2000<-dget("W/seexp/cor_sitc_stack219_2000")#[colnames(seexp)[-30], colnames(seexp)[-30]]
seexp.2001<-dget("W/seexp/cor_sitc_stack219_2001")#[colnames(seexp)[-30], colnames(seexp)[-30]]
seexp.2002<-dget("W/seexp/cor_sitc_stack219_2002")#[colnames(seexp)[-30], colnames(seexp)[-30]]
seexp.2003<-dget("W/seexp/cor_sitc_stack219_2003")#[colnames(seexp)[-30], colnames(seexp)[-30]]
seexp.2004<-dget("W/seexp/cor_sitc_stack219_2004")#[colnames(seexp)[-30], colnames(seexp)[-30]]
seexp.2005<-dget("W/seexp/cor_sitc_stack219_2005")#[colnames(seexp)[-30], colnames(seexp)[-30]]
SE<-list(seexp.1998, seexp.1999, seexp.2000, seexp.2001, seexp.2002, seexp.2003, seexp.2004, seexp.2005)


year.lag<-c(1998, 1999, 2000, 2000, 2002:2004, 2004) ### there are little data for 2001 and 2005, 
                                                     ### so we have to use the data from the previous year. 

CUT<-NULL
for (k in 1:80){
# k<-21
Neib.seexp<-rep(NA, 1*dim(Tax_corp)[1])                      # we don't have data for 1998's lag
for (i in 1:8){se<-SE[[i]]; se[is.na(se)]<-0; diag(se)<-0
               se.ctry<-intersect(rownames(Tax_corp), colnames(se))
               se<-se[se.ctry, se.ctry]
               for (j in 1:dim(se)[1]){se[j,][se[j,]<sort(se[j, ], decreasing = T)[k]]<-0}
               sm<-rowSums(se)
               sm[sm==0]<-1
               se<-se/sm 
               n.se<-se%*%Tax[se.ctry, as.character(year.lag)[i]]
               nse<-NULL
               for (j in 1:dim(Tax_corp)[1]){if (rownames(Tax_corp)[j] %in% rownames(n.se)){nse<-c(nse, n.se[rownames(Tax_corp)[j],])}
                                             else {nse<-c(nse, NA)}}
               Neib.seexp<-c(Neib.seexp, nse)
               }
# 
Neib.seexp[Neib.seexp==0]<-NA # this is the trick here to have better fit for this.

fit.seexp<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                      +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.mdd
                      # +Neib.lng
                      # +Neib.fdi
                      +Neib.seexp
                     )
summary(fit.seexp)
CUT<-rbind(CUT, c(k, summary(fit.seexp)$coef["Neib.seexp",]))
}
CUT

xtable(summary(fit.seexp))




# 4. trade connections: no directionality here:
EX<-dget("W/trade/DOTX8005") 
year.ex<-c(1998:2005)
year.lag<-c(1998:2000, 2000, 2002:2004, 2004)


CUT<-NULL
for (K in c(1, seq(5, 120, by=5))){
Neib.td<-rep(NA, 1*dim(Tax_corp)[1]) # we don't have data for 1998's lag

for (k in 1:8){
tr<-EX[[as.character(year.ex[k])]]
chs<-intersect(rownames(Tax_corp), rownames(tr))
tr<-tr[chs, chs]+t(tr[chs, chs]) # this is total trade and this is symmetric.


for (m in 1:dim(tr)[1]){tr[m,][tr[m,]<sort(as.numeric(tr[m, ]), decreasing = T)[K]]<-0}


w_td<-tr/rowSums(tr, na.rm=T); 
w_td[is.na(w_td)]<-0 # need to do a row-standardization for export data:

w<-matrix(0, ncol=length(rownames(Tax_corp)), nrow=length(rownames(Tax_corp)))
colnames(w)<-rownames(w)<-rownames(Tax_corp)

for (i in 1:dim(w_td)[1])      {
for (j in (1:dim(w_td)[2])[-i]){if (rownames(w_td)[i] %in% rownames(Tax_corp) & colnames(w_td)[j] %in% rownames(Tax_corp)) 
                                     {w[rownames(w_td)[i], colnames(w_td)[j]]<-w_td[i, j]}
                                else {}
                                }}

neib.td<-w%*%Tax[, as.character(year.lag[k])] 

Neib.td<-c(Neib.td, neib.td)
}


fit.td<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                       +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.mdd
                      # +Neib.lng
                       #+Neib.portf
                       #+Neib.fdi
                      #+Neib.seexp
                      +Neib.td
                      #+oecd
                      #+eu
                      # +wto
                     )
summary(fit.td)


CUT<-rbind(CUT, c(K, summary(fit.td)$coef["Neib.td",]))
}

CUT

# do some vertical ropeladders:

va<-as.character(CUT[,1])

op.m<-CUT[,2]
op.se<-CUT[,3]

par(mfrow=c(1,1))
xlim <- c(1,26)
ylim <- c(-.7,.5)

plot(xlim,ylim,type="n",xlab="Choice of K for K nearest neighborhood",ylab="",axes=FALSE, main="");abline(h=0, col="gray")
axis(2);  axis(1, labels=va, at=c(c(2:26)-.1), las=1, padj=0, hadj=0, col="lightgray", tick=F)
segments(2:26, op.m-1.645*op.se, 2:26, op.m+1.645*op.se, col="gray", lwd=2); points(2:26, op.m, pch=19, cex=1)


# 
plot(1:100, 1:100, ylim=c(-.7, 1.6), xlim=c(1, 80), cex.axis=1, las=0, xaxp=c(0,80,8),
     type="n", xlab="Choices of K", ylab="Estimates for Spatial Lag"); abline(h=0, col="gray")
lines(1:80, CUT[,2], lwd=2)
lines(1:80, CUT[,2]-1.6*CUT[,3], col="gray", lwd=2)
lines(1:80, CUT[,2]+1.64*CUT[,3], col="gray", lty=2, lwd=2)


postscript("rho_k.ps", width = 4.0, height = 4.0,
                horizontal = T, 
                family = "Times")
                
dev.off()






## 5. IGO networks:
eu<-dget("W/igo/Eu")
oecd<-dget("W/igo/Oecd")
wto<-dget("W/igo/Wto")

EU<-matrix(0, ncol=dim(Tax_corp)[2], nrow=dim(eu)[1])
colnames(EU)<-colnames(Tax_corp); rownames(EU)<-rownames(eu)
for (j in c(1:3)){for (i in 1:dim(EU)[1]){if (rownames(EU)[i] %in% rownames(eu)[eu[,9+j]==1]){EU[i, j]<-1}}}
EU[, 4]<-EU[, 5]<-EU[, 6]<-EU[, 7]<-EU[, 8]<-EU[, 9]<-EU[, 3]
# new memberships:
# BUL: 2007
# CYP: 2004
# CZR: 2004
# EST: 2004
# HUN: 2004
# LAT: 2004
# LIT: 2004
# MLT: 2004
# POL: 2004
# RUM: 2007
# SLO: 2004
# SLV: 2004
EU["CYP", 7]<-EU["CYP", 8]<-EU["CYP", 9]<-1
EU["CZR", 7]<-EU["CZR", 8]<-EU["CZR", 9]<-1
EU["EST", 7]<-EU["EST", 8]<-EU["EST", 9]<-1
EU["HUN", 7]<-EU["HUN", 8]<-EU["HUN", 9]<-1
EU["LAT", 7]<-EU["LAT", 8]<-EU["LAT", 9]<-1
EU["LIT", 7]<-EU["LIT", 8]<-EU["LIT", 9]<-1
EU["MLT", 7]<-EU["MLT", 8]<-EU["MLT", 9]<-1
EU["POL", 7]<-EU["POL", 8]<-EU["POL", 9]<-1
EU["SLO", 7]<-EU["SLO", 8]<-EU["SLO", 9]<-1
EU["SLV", 7]<-EU["SLV", 8]<-EU["SLV", 9]<-1


OECD<-matrix(0, ncol=dim(Tax_corp)[2], nrow=dim(oecd)[1])
colnames(OECD)<-colnames(Tax_corp); rownames(OECD)<-rownames(oecd)
for (j in c(1:3)){for (i in 1:dim(OECD)[1]){if (rownames(OECD)[i] %in% rownames(oecd)[oecd[,9+j]==1]){OECD[i, j]<-1}}}
OECD[, 4]<-OECD[, 5]<-OECD[, 6]<-OECD[, 7]<-OECD[, 8]<-OECD[, 9]<-OECD[, 3]
# new membership
# SLO: dec, 2000
OECD["SLO", 4]<-OECD["SLO", 5]<-OECD["SLO", 6]<-OECD["SLO", 7]<-OECD["SLO", 8]<-OECD["SLO", 9]<-1


WTO<-matrix(0, ncol=dim(Tax_corp)[2], nrow=dim(wto)[1])
colnames(WTO)<-colnames(Tax_corp); rownames(WTO)<-rownames(wto)
for (j in c(1:3)){for (i in 1:dim(WTO)[1]){if (rownames(WTO)[i] %in% rownames(wto)[wto[,9+j]==1]){WTO[i, j]<-1}}}
WTO[, 4]<-WTO[, 5]<-WTO[, 6]<-WTO[, 7]<-WTO[, 8]<-WTO[, 9]<-WTO[, 3]
# new membership
# ALB: 2000
# ARM: 2003
# CRO: 2000
# MAC: 2003
# JOR: 2000
# MLD: 2001
# NEP: 2004
# SAU: 2005
# OMA: 2000
# GRG: 2000
# LIT: 2001
# CAM: 2004
# CHN: 2001
WTO["ARM", 6]<-WTO["ARM", 7]<-WTO["ARM", 8]<-WTO["ARM", 9]<-1
WTO["MAC", 6]<-WTO["MAC", 7]<-WTO["MAC", 8]<-WTO["MAC", 9]<-1
WTO["MLD", 4]<-WTO["MLD", 5]<-WTO["MLD", 6]<-WTO["MLD", 7]<-WTO["MLD", 8]<-WTO["MLD", 9]<-1
WTO["NEP", 7]<-WTO["NEP", 8]<-WTO["NEP", 9]<-1
WTO["SAU", 8]<-WTO["SAU", 9]<-1
WTO["LIT", 4]<-WTO["LIT", 5]<-WTO["LIT", 6]<-WTO["LIT", 7]<-WTO["LIT", 8]<-WTO["LIT", 9]<-1
WTO["CAM", 7]<-WTO["CAM", 8]<-WTO["CAM", 9]<-1
WTO["CHN", 4]<-WTO["CHN", 5]<-WTO["CHN", 6]<-WTO["CHN", 7]<-WTO["CHN", 8]<-WTO["CHN", 9]<-1


Temp<-WTO
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
wto<-iv; wto.lg<-iv.lg

Temp<-EU
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
eu<-iv; eu.lg<-iv.lg

Temp<-OECD
iv<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]) %in% colnames(Temp)){iv<-c(iv, Temp[cty[i], as.character(year[i])])}
                              else {iv<-c(iv, NA)}}
iv.lg<-NULL
for (i in 1:length(dv.tax.c)){if (cty[i] %in% rownames(Temp) & as.character(year[i]-1) %in% colnames(Temp)){iv.lg<-c(iv.lg, Temp[cty[i], as.character(year[i]-1)])}
                              else {iv.lg<-c(iv.lg, NA)}
                              }
oecd<-iv; oecd.lg<-iv.lg


#
igo.em<-dget("W/igo/Matrix.igo.em.8900")
igo.sm<-dget("W/igo/Matrix.igo.8900")

igo1998<-dget("W/igo/igo1998")
igo1999<-dget("W/igo/igo1999")
igo2000<-dget("W/igo/igo2000")
# IGO<-list(igo1998, igo1999, igo2000)
# IGO<-list(igo.sm[["1998"]], igo.sm[["1999"]], igo.sm[["2000"]])
 IGO<-list(igo.em[["1998"]], igo.em[["1999"]], igo.em[["2000"]])

year.lag<-c(1998:2000)





CUT<-NULL
for (K in c(1, seq(5, 120, by=5))){

Neib.igo<-rep(NA, 1*dim(Tax_corp)[1]) # we don't have data for 1998's lag

for (k in 1:3){
igo<-IGO[[k]]
chs<-intersect(rownames(Tax_corp), rownames(igo))
# chs<-intersect(rownames(igo), rownames(igo))

for (m in 1:dim(igo)[1]){igo[m,][igo[m,]<sort(as.numeric(igo[m, ]), decreasing = T)[K]]<-0}


w_igo<-igo[chs, chs]/rowSums(igo[chs, chs], na.rm=T); 
w_igo[is.na(w_igo)]<-0 # need to do a row-standardization for export data:

w<-matrix(0, ncol=length(rownames(Tax_corp)), nrow=length(rownames(Tax_corp)))
colnames(w)<-rownames(w)<-rownames(Tax_corp)

for (i in 1:dim(w_igo)[1])      {
for (j in (1:dim(w_igo)[2])[-i]){if (rownames(w_igo)[i] %in% rownames(Tax_corp) & colnames(w_igo)[j] %in% rownames(Tax_corp)) 
                                     {w[rownames(w_igo)[i], colnames(w_igo)[j]]<-w_igo[i, j]}
                                else {}
                                }}

neib.igo<-w%*%Tax[, as.character(year.lag[k])] 

Neib.igo<-c(Neib.igo, neib.igo)
}

Neib.igo<-c(Neib.igo, rep(NA, 5*dim(Tax_corp)[1])) # we don't have data for 1998's lag


# plot(Neib.igo, dv.tax.c) 
fit.igo<-lm(dv.tax.c~dv.tax.c.lg
                      +as.factor(cty)
                      +as.factor(year)
                      +infl
                      +grow
                       +unemp
                      +I(dp65+dp14)
                      +gdppc
                      +capopen
                      +tradeopen
                      +Neib.mdd
                      #+Neib.lng
                      # +Neib.portf
                      # +Neib.fdi
                      #+Neib.seexp
                       +Neib.igo
                      #+ oecd 
                      #+ eu
                      #+ wto
                      #+Neib.td
                     , na.action=na.omit)
summary(fit.igo)


CUT<-rbind(CUT, c(K, summary(fit.igo)$coef["Neib.igo",]))
}

CUT

va<-as.character(CUT[,1])

op.m<-CUT[,2]
op.se<-CUT[,3]

par(mfrow=c(1,1))
xlim <- c(1,26)
ylim <- c(-.2,6.8)

plot(xlim,ylim,type="n",xlab="Choice of K for K nearest neighborhood",ylab="",axes=FALSE, main="");abline(h=0, col="gray")
axis(2);  axis(1, labels=va, at=c(c(2:26)-.1), las=1, padj=0, hadj=0, col="lightgray", tick=F)
segments(2:26, op.m-1.645*op.se, 2:26, op.m+1.645*op.se, col="gray", lwd=2); points(2:26, op.m, pch=19, cex=1)



### stop here:
# notice that there are zeros in weights, but mostly excluded for missing values.
